@@ -27,7 +27,7 @@ import click
 from click import echo, style, secho, getchar
 from click.termui import strip_ansi
 
-__version__ = '2.2.1'
+__version__ = '2.3.0'
 __author__ = 'Steven Loria'
 __license__ = 'MIT'
 
@@ -43,13 +43,13 @@ else:
 THEMES = OrderedDict([
     ('default', '{user.cyan.bold}@{hostname.blue}:{dir.green} $'),
 
-    ('sorin', '{cwd.cyan} {git_branch.green.git} '
-        '{r_angle.red}{r_angle.yellow}{r_angle.green}'),
+    ('sorin', '{cwd.cyan} {vcs_branch.green.git} '
+     '{r_angle.red}{r_angle.yellow}{r_angle.green}'),
 
     ('nicolauj', '{r_angle.white}'),
 
     ('steeef', '{user.red} at {hostname.yellow} in {cwd.green} '
-                '{git_branch.cyan.paren}\n$'),
+               '{vcs_branch.cyan.paren}\n$'),
 
     ('redhat', '[{user}@{hostname} {dir}]$'),
     ('redhat_color', '[{user.red.bold}@{hostname.red} {dir.blue}]$'),
@@ -57,18 +57,18 @@ THEMES = OrderedDict([
     ('walters', '{user}@{hostname.underlined}>'),
     ('walters_color', '{user.cyan.bold}@{hostname.blue.underlined}>'),
 
-    ('minimal', '{dir} {git_branch.square} »'),
-    ('minimal_color', '{dir.cyan} {git_branch.blue.square} »'),
+    ('minimal', '{dir} {vcs_branch.square} »'),
+    ('minimal_color', '{dir.cyan} {vcs_branch.blue.square} »'),
 
     ('osx', '{hostname}:{dir} {user}$'),
     ('osx_color', '{hostname.blue}:{dir.green} {user.cyan}$'),
 
     ('pws', '{TTY.BOLD}+{TTY.YELLOW}{now:%I:%M}{TTY.RESET}%'),
 
-    ('robbyrussell', '{r_arrow.red} {dir.cyan} {git_branch.red.paren.git}'),
+    ('robbyrussell', '{r_arrow.red} {dir.cyan} {vcs_branch.red.paren.git}'),
 
     ('giddie', '{user.magenta}@{hostname.yellow}|{cwd.green} '
-                'on {git_branch.magenta}\n{TTY.BLUE}±{TTY.RESET}')
+                'on {vcs_branch.magenta}\n{TTY.BLUE}±{TTY.RESET}')
 
 ])
 
@@ -76,9 +76,8 @@ THEMES = OrderedDict([
 ESC = '\x1b'
 RETURNS = {'\r', '\n'}
 OPTION_RE = re.compile(r'^#\s?doitlive\s+'
-            '(?P<option>prompt|shell|alias|env|speed|unalias|unset):'
-            '\s*(?P<arg>.+)$')
-
+                       '(?P<option>prompt|shell|alias|env|speed'
+                       '|unalias|unset|commentecho):\s*(?P<arg>.+)$')
 
 TESTING = False
 
@@ -135,13 +134,20 @@ class TermString(unicode):
     def curly(self):
         return self._bracketed('{', '}')
 
-    @property
-    def git(self):
+    def _vcs(self, vcsname):
         if strip_ansi(self):
-            return TermString('{}:{}'.format(style('git', fg='blue'), self))
+            return TermString('{}:{}'.format(style(vcsname, fg='blue'), self))
         else:
             return TermString('\b')
 
+    @property
+    def git(self):
+        return self._vcs('git')
+
+    @property
+    def hg(self):
+        return self._vcs('hg')
+
 
 class ANSICode(object):
     """Descriptor that returns the ANSI code the given styles passed to
@@ -178,7 +184,8 @@ class TTY(object):
 def get_current_git_branch():
     command = ['git', 'symbolic-ref', '--short', '-q', 'HEAD']
     try:
-        proc = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        proc = subprocess.Popen(command, stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE)
         out, _ = proc.communicate()
         return out.strip().decode('utf-8')
     except subprocess.CalledProcessError:
@@ -186,6 +193,64 @@ def get_current_git_branch():
     return ''
 
 
+# We'll avoid shelling out to hg for speed.
+def find_hg_root():
+    def get_parent_dir(d):
+        return os.path.abspath(os.path.join(d, os.pardir))
+
+    cwd = os.getcwd()
+    while True:
+        pardir = get_parent_dir(cwd)
+
+        hgroot = os.path.join(cwd, '.hg')
+        if os.path.isdir(hgroot):
+            return hgroot
+
+        if cwd == pardir:
+            break
+
+        cwd = pardir
+
+    return ''
+
+
+def get_current_hg_branch():
+    try:
+        hgroot = find_hg_root()
+        with open(os.path.join(hgroot, 'branch')) as f:
+            branch = f.read().rstrip()
+    except IOError:
+        branch = ''
+
+    return branch
+
+
+def get_current_hg_bookmark():
+    try:
+        hgroot = find_hg_root()
+        with open(os.path.join(hgroot, 'bookmarks.current')) as f:
+            bookmark = f.read()
+    except IOError:
+        bookmark = ''
+    return bookmark
+
+
+def get_current_hg_id():
+    branch = get_current_hg_branch()
+    bookmark = get_current_hg_bookmark()
+    if bookmark:
+        # If we have a bookmark, the default branch is no longer
+        # an interesting name.
+        if branch == "default":
+            branch = ""
+        branch += " " + bookmark
+    return branch
+
+
+def get_current_vcs_branch():
+    return get_current_git_branch() + get_current_hg_id()
+
+
 # Some common symbols used in prompts
 R_ANGLE = TermString('❯')
 R_ANGLE_DOUBLE = TermString('»')
@@ -204,6 +269,10 @@ def get_prompt_state():
         'dir': TermString(dir_raw),
         'hostname': TermString(socket.gethostname()),
         'git_branch': TermString(get_current_git_branch()),
+        'hg_id': TermString(get_current_hg_id()),
+        'hg_branch': TermString(get_current_hg_branch()),
+        'hg_bookmark': TermString(get_current_hg_bookmark()),
+        'vcs_branch': TermString(get_current_vcs_branch()),
         # Symbols
         'r_angle': R_ANGLE,
         'r_angle_double': R_ANGLE_DOUBLE,
@@ -295,10 +364,10 @@ def magictype(text, prompt_template='default', speed=1):
 
 
 def magicrun(text, shell, prompt_template='default', aliases=None,
-        envvars=None, speed=1, test_mode=False):
+             envvars=None, speed=1, test_mode=False, commentecho=False):
     magictype(text, prompt_template, speed)
     run_command(text, shell, aliases=aliases, envvars=envvars,
-        test_mode=test_mode)
+                test_mode=test_mode)
 
 
 def format_prompt(prompt):
@@ -379,7 +448,8 @@ class PythonPlayerConsole(InteractiveConsole):
             sys.ps2
         except AttributeError:
             sys.ps2 = "... "
-        cprt = 'Type "help", "copyright", "credits" or "license" for more information.'
+        cprt = ('Type "help", "copyright", "credits" or "license" for '
+                'more information.')
         if banner is None:
             self.write("Python %s on %s\n%s\n" %
                        (sys.version, sys.platform, cprt))
@@ -407,12 +477,13 @@ class SessionState(dict):
     """Stores information about a fake terminal session."""
 
     def __init__(self, shell, prompt_template, speed,
-                aliases=None, envvars=None,
-                test_mode=False):
+                 aliases=None, envvars=None,
+                 test_mode=False, commentecho=False):
         aliases = aliases or []
         envvars = envvars or []
         dict.__init__(self, shell=shell, prompt_template=prompt_template,
-            speed=speed, aliases=aliases, envvars=envvars, test_mode=test_mode)
+                      speed=speed, aliases=aliases, envvars=envvars,
+                      test_mode=test_mode, commentecho=commentecho)
 
     def add_alias(self, alias):
         self['aliases'].append(alias)
@@ -443,6 +514,13 @@ class SessionState(dict):
     def remove_envvar(self, envvar):
         return self._remove_var('envvars', envvar)
 
+    def commentecho(self, doit=None):
+        if doit is not None:
+            doit = doit.lower()
+            self['commentecho'] = (doit == 'true' or doit == 'yes'
+                                     or doit == '1')
+        return self['commentecho']
+
 # Map of option names => function that modifies session state
 OPTION_MAP = {
     'prompt': lambda state, arg: state.set_template(arg),
@@ -452,19 +530,22 @@ OPTION_MAP = {
     'speed': lambda state, arg: state.set_speed(arg),
     'unalias': lambda state, arg: state.remove_alias(arg),
     'unset': lambda state, arg: state.remove_envvar(arg),
+    'commentecho': lambda state, arg: state.commentecho(arg),
 }
 
 
 def run(commands, shell='/bin/bash', prompt_template='default', speed=1,
-        test_mode=False):
-    secho("We'll do it live!", fg='red', bold=True)
-    secho('STARTING SESSION: Press Ctrl-C at any time to exit.',
-        fg='yellow', bold=True)
+        quiet=False, test_mode=False, commentecho=False):
+    if not quiet:
+        secho("We'll do it live!", fg='red', bold=True)
+        secho('STARTING SESSION: Press Ctrl-C at any time to exit.',
+              fg='yellow', bold=True)
+        click.pause()
 
-    click.pause()
     click.clear()
     state = SessionState(shell=shell, prompt_template=prompt_template,
-        speed=speed, test_mode=test_mode)
+                         speed=speed, test_mode=test_mode,
+                         commentecho=commentecho)
 
     i = 0
     while i < len(commands):
@@ -479,6 +560,9 @@ def run(commands, shell='/bin/bash', prompt_template='default', speed=1,
                 option, arg = match.group('option'), match.group('arg')
                 func = OPTION_MAP[option]
                 func(state, arg)
+            elif state.commentecho():
+                comment = command.lstrip("#")
+                secho(comment, fg='yellow', bold=True)
             continue
         elif command.startswith('```python'):
             py_commands = []
@@ -487,7 +571,8 @@ def run(commands, shell='/bin/bash', prompt_template='default', speed=1,
                 try:
                     py_command = commands[i].rstrip()
                 except IndexError:
-                    raise SessionError('Unmatched python code block in session file.')
+                    raise SessionError('Unmatched python code block in '
+                                       'session file.')
                 i += 1
                 if py_command.startswith('```'):
                     i += 1
@@ -557,16 +642,25 @@ def themes(preview, list):
     else:
         list_themes()
 
+QUIET_OPTION = click.option('--quiet', '-q', help='Suppress startup message.',
+                            is_flag=True, default=False, show_default=False)
+
+ECHO_OPTION = click.option('--commentecho', '-e',
+                           help='Echo non-magic comments.', is_flag=True,
+                           default=False, show_default=False)
+
 SHELL_OPTION = click.option('--shell', '-S', metavar='<shell>',
-        default='/bin/bash', help='The shell to use.', show_default=True)
+                            default='/bin/bash', help='The shell to use.',
+                            show_default=True)
 
 SPEED_OPTION = click.option('--speed', '-s', metavar='<int>', default=1,
-        help='Typing speed.', show_default=True)
+                            help='Typing speed.', show_default=True)
 
 PROMPT_OPTION = click.option('--prompt', '-p', metavar='<prompt_theme>',
-        default='default', type=click.Choice(THEMES.keys()),
-        help='Prompt theme.',
-        show_default=True)
+                             default='default',
+                             type=click.Choice(THEMES.keys()),
+                             help='Prompt theme.',
+                             show_default=True)
 
 ALIAS_OPTION = click.option('--alias', '-a', metavar='<alias>',
     multiple=True, help='Add a session alias.')
@@ -581,20 +675,24 @@ def _compose(*functions):
     return functools.reduce(inner, functions)
 
 # Compose the decorators into "bundled" decorators
-player_command = _compose(SHELL_OPTION, SPEED_OPTION, PROMPT_OPTION)
-recorder_command = _compose(SHELL_OPTION, PROMPT_OPTION, ALIAS_OPTION, ENVVAR_OPTION)
+player_command = _compose(QUIET_OPTION, SHELL_OPTION, SPEED_OPTION,
+                          PROMPT_OPTION, ECHO_OPTION)
+recorder_command = _compose(SHELL_OPTION, PROMPT_OPTION, ALIAS_OPTION,
+                            ENVVAR_OPTION)
 
 
 @player_command
 @click.argument('session_file', type=click.File('r', encoding='utf-8'))
 @cli.command()
-def play(session_file, shell, speed, prompt):
+def play(quiet, session_file, shell, speed, prompt, commentecho):
     """Play a session file."""
     run(session_file.readlines(),
         shell=shell,
         speed=speed,
+        quiet=quiet,
         test_mode=TESTING,
-        prompt_template=prompt)
+        prompt_template=prompt,
+        commentecho=commentecho)
 
 DEMO = [
     'echo "Greetings"',
@@ -606,9 +704,10 @@ DEMO = [
 
 @player_command
 @cli.command()
-def demo(shell, speed, prompt):
+def demo(quiet, shell, speed, prompt, commentecho):
     """Run a demo doitlive session."""
-    run(DEMO, shell=shell, speed=speed, test_mode=TESTING, prompt_template=prompt)
+    run(DEMO, shell=shell, speed=speed, test_mode=TESTING,
+        prompt_template=prompt, quiet=quiet, commentecho=commentecho)
 
 
 HEADER_TEMPLATE = """# Recorded with the doitlive recorder
@@ -643,7 +742,8 @@ def run_recorder(shell, prompt, aliases=None, envvars=None):
         elif command == PREVIEW_COMMAND:
             echo_rec_buffer(commands)
         elif command == UNDO_COMMAND:
-            if commands and click.confirm('Remove command? "{}"'.format(commands[-1].strip())):
+            if commands and click.confirm(
+                    'Remove command? "{}"'.format(commands[-1].strip())):
                 commands.pop()
                 secho('Removed command.', bold=True)
                 echo_rec_buffer(commands)
@@ -696,7 +796,7 @@ def record(session_file, shell, prompt, alias, envvar):
     secho("We'll do it live!", fg='red', bold=True)
     filename = click.format_filename(session_file)
     secho('RECORDING SESSION: {}'.format(filename),
-        fg='yellow', bold=True)
+          fg='yellow', bold=True)
 
     print_recorder_instructions()
 
